;Debug library for using with EMUDBG.EXE.
;
;In main program you must define:
;
;1. 2 bytes in Internal Data Memory as debugger variables:
;
;DBGVA	.EQU 030H ;debugger variable address
;DBGVV	.EQU 031H ;debugger variable value
;
;2. Debugger address in External Data Memory for emulator change register:
;
;DBGA	.EQU 0000H ;debugger address (16 bit)
;
;Library file "LIBDEF.ASM" includes all definations for debugging.
;
;First you need is debug init at beginning of your program.
;For example:
;
;INIT:	MOV SP,#STACK
;	DEBUGINIT
;
;At any point of your program you can use 3 debug operators:
;
;	PRINT(VAR) ;prints variable address and value
;
;	MODIFY(VAR) ;modifies variable value
;
;	BREAKPOINT ;shows breakpoint address and
;		   ;stops the program until you press F1 key
;
;As VAR you can use:
;1. Internal memory variable name or address
;2. Special function register name or address
;3. Register R0-R7 address (not name!)
;
;Debug subroutines

DBGI:	PUSH ACC
	PUSH DPL
	PUSH DPH
	MOV DPTR,#DBGA
	MOV A,#0FFH
	MOVX @DPTR,A
	POP DPH
	POP DPL
	POP ACC
	RET

DBGP:	PUSH ACC
	PUSH DPL
	PUSH DPH

	MOV DPTR,#DBGA
	MOVX A,@DPTR
	CPL A
	JZ DBG1          ;jump if debug off

	MOV A,DBGVA
	SWAP A
	LCALL DBG2

	MOV A,DBGVA
	LCALL DBG2

	MOV A,DBGVV
	SWAP A
	LCALL DBG2

	MOV A,DBGVV
	LCALL DBG2

DBG1:	POP DPH
	POP DPL
	POP ACC
	RET

DBGB:	CLR EA
	POP DBGVA
	POP DBGVV
	INC SP
	INC SP
	PUSH ACC
	PUSH DPL
	PUSH DPH

	MOV DPTR,#DBGA
	MOVX A,@DPTR
	CPL A
	JZ DBG10          ;jump if debug off

DBG7:	MOVX A,@DPTR
	CJNE A,#0FEH,DBG7 ;wait for IBM ready
	MOV A,#0FEH
	MOVX @DPTR,A       ;write breakpoint flag
DBG8:	MOVX A,@DPTR
	ANL A,#0FDH
	CJNE A,#0FDH,DBG8
	MOV A,#0FFH
	MOVX @DPTR,A

	MOV A,DBGVA
	SWAP A
	LCALL DBG2

	MOV A,DBGVA
	LCALL DBG2

	MOV A,DBGVV
	SWAP A
	LCALL DBG2

	MOV A,DBGVV
	LCALL DBG2

DBG6:	MOVX A,@DPTR
	CJNE A,#0FFH,DBG9
	SJMP DBG10
DBG9:	CJNE A,#0FBH,DBG6

DBG10:	SETB EA

	POP DPH
	POP DPL
	POP ACC
	RET

DBGM:	PUSH ACC
	PUSH DPL
	PUSH DPH

	MOV DPTR,#DBGA
	MOVX A,@DPTR
	CPL A
	JZ DBG11         ;jump if debug off

DBG12:	MOVX A,@DPTR
	CJNE A,#0FEH,DBG12 ;wait for IBM ready
	MOV A,#0FDH
	MOVX @DPTR,A        ;write modify flag
DBG13:	MOVX A,@DPTR
	ANL A,#0FDH
	CJNE A,#0FDH,DBG13
	MOV A,#0FFH
	MOVX @DPTR,A

	MOV A,DBGVA
	SWAP A
	LCALL DBG2

	MOV A,DBGVA
	LCALL DBG2

	MOV A,DBGVV
	SWAP A
	LCALL DBG2

	MOV A,DBGVV
	LCALL DBG2

DBG14:	MOVX A,@DPTR
	CJNE A,#0FFH,DBG15
	SJMP DBG11
DBG15:	JB ACC.7,DBG14
	MOVX A,@DPTR
	SWAP A
	MOV DBGVV,A
	MOV A,#0FCH
	MOVX @DPTR,A
DBG16:	MOVX A,@DPTR
	ANL A,#0FDH
	CJNE A,#0FDH,DBG16
	MOV A,#0FFH
	MOVX @DPTR,A
DBG17:	MOVX A,@DPTR
	JB ACC.7,DBG17
	MOVX A,@DPTR
	ORL A,DBGVV
	MOV DBGVV,A
	MOV A,#0FCH
	MOVX @DPTR,A
DBG18:	MOVX A,@DPTR
	ANL A,#0FDH
	CJNE A,#0FDH,DBG18
	MOV A,#0FFH
	MOVX @DPTR,A

DBG11:	POP DPH
	POP DPL
	POP ACC
	RET

DBG2:	PUSH ACC
DBG3:	MOVX A,@DPTR
	CJNE A,#0FEH,DBG3 ;wait for IBM ready
	POP ACC
	ANL A,#0FH
	MOVX @DPTR,A
DBG4:	MOVX A,@DPTR
	ANL A,#0FDH
	CJNE A,#0FDH,DBG4
	MOV A,#0FFH
	MOVX @DPTR,A
	RET

